module.exports = {
  parser: '@typescript-eslint/parser',
  extends: [
    'airbnb',
    'prettier',
    'prettier/@typescript-eslint',
    'plugin:compat/recommended',
    'plugin:@typescript-eslint/recommended',
  ],
  plugins: ['@typescript-eslint', 'react-hooks'],
  env: {
    browser: true,
    node: true,
    es6: true,
    mocha: true,
    jest: true,
    jasmine: true,
  },
  globals: {
    WODAX_BUILD_APPLICATION_NAME: true,
    WODAX_BUILD_APPLICATION_VERSION: true,
    WODAX_BUILD_ENV: true,
    WODAX_BUILD_DATETIME: true,
    WODAX_BUILD_GIT_URL: true,
    WODAX_BUILD_GIT_BRANCH: true,
    WODAX_BUILD_GIT_COMMIT_ID: true,

    WODAX_APP_NAME: true,
    WODAX_APP_VERSION: true,
    __ALIYUN_ENV__: true,
    APP_TYPE: true,

    page: true,
  },
  rules: {
    /// for base 配置
    'generator-star-spacing': 0,
    'function-paren-newline': 0,
    'linebreak-style': 0,
    'no-extra-boolean-cast': 1,
    'no-unused-vars': 0,
    'no-script-url': 0,
    'no-param-reassign': 0,
    'no-underscore-dangle': 0,
    'no-nested-ternary': 1,
    'no-restricted-syntax': ['off'],
    'no-prototype-builtins': 0,
    'no-plusplus': [1, { allowForLoopAfterthoughts: true }],
    // Use function hoisting to improve code readability
    'no-use-before-define': [
      'error',
      { functions: false, classes: true, variables: true },
    ],

    /// for compat
    'compat/compat': 0,
    // Common abbreviations are known and readable
    'unicorn/prevent-abbreviations': 'off',

    /// for import 配置
    'import/no-cycle': 0,
    'import/no-unresolved': [
      2,
      { ignore: ['^@/', '^umi/', '^./', '^../', '^.', '^..'] },
    ],
    'import/no-extraneous-dependencies': 0,
    'import/prefer-default-export': 0,
    'import/no-default-export': [0, 'camel-case'],
    'import/extensions': 0,

    'prefer-destructuring': ['error', { object: true, array: false }],

    /// for jsx-a11y 配置
    'jsx-a11y/no-noninteractive-element-interactions': 0,
    'jsx-a11y/click-events-have-key-events': 0,
    'jsx-a11y/no-static-element-interactions': 0,
    'jsx-a11y/anchor-is-valid': 0,
    'jsx-a11y/label-has-for': [
      1,
      {
        /// for < esLint@^6.1.0
        components: ['Label'],
        required: {
          every: ['nesting', 'id'],
        },
        allowChildren: false,
      },
    ],
    'jsx-a11y/label-has-associated-control': [
      /// for esLint@^6.1.0
      1,
      {
        labelComponents: ['CustomInputLabel'],
        labelAttributes: ['label'],
        controlComponents: ['CustomInput'],
        depth: 3,
      },
    ],

    /// for react/jsx 配置
    'react/no-array-index-key': 1,
    'react/no-unused-state': 0,
    'react/jsx-filename-extension': [
      1,
      { extensions: ['.js', '.jsx', '.tsx'] },
    ],
    'react/jsx-wrap-multilines': 0,
    'react/jsx-boolean-value': 0,
    'react/prop-types': 0,
    'react/prefer-stateless-function': 1,
    'react/forbid-prop-types': 0,
    'react/jsx-no-bind': 0,
    'react/jsx-props-no-spreading': 0,
    'react/jsx-fragments': 0,
    'react/jsx-curly-newline': 0,
    'react/jsx-one-expression-per-line': 0,
    'react/jsx-tag-spacing': [1, { beforeSelfClosing: 'always' }],
    // Too restrictive: https://github.com/yannickcr/eslint-plugin-react/blob/master/docs/rules/destructuring-assignment.md
    'react/destructuring-assignment': 0,

    'react-hooks/rules-of-hooks': 'error', // Checks rules of Hooks
    // issue https://github.com/facebook/react/issues/15204
    'react-hooks/exhaustive-deps': 0, // Checks effect dependencies

    // ==> @typescript-eslint 配置
    '@typescript-eslint/camelcase': 0,
    '@typescript-eslint/indent': 0,
    '@typescript-eslint/interface-name-prefix': 0,
    '@typescript-eslint/explicit-member-accessibility': 0,
    '@typescript-eslint/no-empty-function': 0,
    '@typescript-eslint/no-explicit-any': 0,
    '@typescript-eslint/no-inferrable-types': 0,
    // Makes no sense to allow type inferrence for expression parameters, but require typing the response
    '@typescript-eslint/explicit-function-return-type': [
      'off',
      { allowTypedFunctionExpressions: true },
    ],
    '@typescript-eslint/no-use-before-define': [
      'error',
      { functions: false, classes: true, variables: true, typedefs: true },
    ],
    '@typescript-eslint/no-var-requires': 0,
    '@typescript-eslint/no-unused-vars': 0,
    '@typescript-eslint/ban-ts-ignore': 0,
    '@typescript-eslint/no-non-null-assertion': 0,

    // ==> Conflict with prettier
    'arrow-body-style': [1, 'as-needed'],
    'arrow-parens': 1,
    'object-curly-newline': 0,
    'implicit-arrow-linebreak': 0,
    'operator-linebreak': 0,
    'space-before-function-paren': 0,
  },
  settings: {
    // support import modules from TypeScript files in JavaScript files
    'import/resolver': { node: { extensions: ['.js', '.jsx', '.ts', '.tsx'] } },
    polyfills: ['fetch', 'promises', 'url', 'Promise', 'URL', 'object-assign'],
  },
};
