/*---------------------------------------------------------------------------------------------
 *  Copyright (c) wodax Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
'use strict';
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var util = require('util');
var fs = require('fs');
var fse = require('fs-extra');
var path = require('path');
var _ = require('lodash');
var chalk = require('chalk');
var fmt = require('format-package');
var enableDebug = true;
var PATCH_FILES_DIR_NAME = "patch-files";
var WODAX_PROJECT_DIR_NAME = "wodax-project";
var WODAX_CORE_H5_PROJECT_DIR_NAME = "wodax-core-h5";
var PACKAGES_DIR_NAME = "packages";
var ROOT_PROJECT_PATCH_DIR_NAME = "root";
/** 获取当前时间
 * @example 2019-7-11 17:11:54
 * @returns {string}
 */
function getNowDate() {
    return (new Date()).toLocaleString();
}
var traceStartCommand = function (command, isSpecCommand) {
    if (isSpecCommand === void 0) { isSpecCommand = false; }
    if (!!!enableDebug)
        return;
    try {
        var commandPrefix = isSpecCommand ? "[\u6267\u884C\u8BED\u53E5]" : "[\u547D\u4EE4\u5F00\u59CB]";
        console.log(chalk.gray(">>>" + commandPrefix) + " " + command + "  --- " + getNowDate());
    }
    catch (e) {
    }
};
var traceEndCommand = function (command) {
    if (!!!enableDebug)
        return;
    try {
        console.log(chalk.gray('<<<[命令结束]') + " " + command + "  --- " + getNowDate());
    }
    catch (e) {
    }
};
var traceRunError = function (e) {
    if (!!!enableDebug)
        return;
    var errInfo = "";
    try {
        errInfo = util.inspect(e, { depth: 5, color: true, compact: false });
    }
    catch (e) {
        console.error(e);
    }
    console.error(chalk.red('检测到错误:') + " " + errInfo + "  --- " + getNowDate());
};
var traceWarning = function (prefix, content) {
    if (!!!enableDebug)
        return;
    try {
        console.log(chalk.yellow(prefix) + " " + content);
    }
    catch (e) {
    }
};
var traceOutContent = function (prefix, content) {
    if (!!!enableDebug)
        return;
    try {
        console.log(chalk.gray(prefix) + " " + content);
    }
    catch (e) {
    }
};
/**
 * 严格模式，该方式不可使用
 * @param callee
 */
var getFnName = function (callee) {
    var _callee = callee.toString().replace(/[\s\?]*/g, ""), comb = _callee.length >= 50 ? 50 : _callee.length;
    _callee = _callee.substring(0, comb);
    var name = _callee.match(/^function([^\(]+?)\(/);
    if (name && name[1]) {
        return name[1];
    }
    var caller = callee.caller, _caller = caller.toString().replace(/[\s\?]*/g, "");
    var last = _caller.indexOf(_callee), str = _caller.substring(last - 30, last);
    name = str.match(/var([^\=]+?)\=/);
    if (name && name[1]) {
        return name[1];
    }
    return "anonymous";
};
var getCurrentPackageVersion = function () {
    var _version = "latest";
    try {
        var specPKG = require("./package");
        var version = specPKG.version;
        if (version && (typeof version === 'string')) {
            _version = version;
        }
    }
    catch (e) { }
    return _version;
};
var ActionType;
(function (ActionType) {
    ActionType[ActionType["addOrUpdateFolderFile"] = 0] = "addOrUpdateFolderFile";
    ActionType[ActionType["deleteFolderFile"] = 1] = "deleteFolderFile";
    ActionType[ActionType["contentAdd"] = 2] = "contentAdd";
    ActionType[ActionType["contentReplace"] = 3] = "contentReplace";
    ActionType[ActionType["contentReplaceX"] = 4] = "contentReplaceX";
    ActionType[ActionType["contentRemove"] = 5] = "contentRemove";
    ActionType[ActionType["rename"] = 6] = "rename";
    ActionType[ActionType["gitAdd"] = 7] = "gitAdd";
})(ActionType = exports.ActionType || (exports.ActionType = {}));
var PatchHelper = /** @class */ (function () {
    function PatchHelper(_a) {
        var _name = _a._name, _patchRootDir = _a._patchRootDir, _projectDir = _a._projectDir;
        this.name = _name;
        this.patchRootDir = _patchRootDir;
        this.projectDir = _projectDir;
        traceOutContent("name", "" + this.name);
        traceOutContent("patchRootDir", "" + this.patchRootDir);
        traceOutContent("projectDir", "" + this.projectDir);
    }
    PatchHelper.prototype.callAction = function (_a) {
        var action = _a.action, options = _a.options;
        if (action === ActionType.addOrUpdateFolderFile) {
        }
    };
    PatchHelper.prototype.commonProcessForFolderFile = function (_a) {
        var _this = this;
        var action = _a.action, options = _a.options;
        traceStartCommand("commonProcessForFolderFile");
        if (ActionType.addOrUpdateFolderFile === action) {
            var files = fs.readdirSync(this.patchRootDir);
            files.forEach(function (onePath) {
                traceOutContent("\u68C0\u6D4B\u6D4B\u8BD5\u8DEF\u5F84", "" + onePath);
                var src = "" + path.normalize(path.join(_this.patchRootDir, onePath));
                var dest = "" + path.normalize(path.join(_this.projectDir, onePath));
                fse.copySync(src, dest, {
                    overWrite: true
                });
            });
        }
        else if (ActionType.deleteFolderFile === action) {
            var filePath = options.filePath;
            var dest = "" + path.normalize(path.join(this.projectDir, filePath));
            fse.removeSync(dest);
        }
        traceEndCommand("commonProcessForFolderFile");
    };
    PatchHelper.prototype.commonGitAction = function (_a) {
        var action = _a.action, options = _a.options;
        traceStartCommand("commonGitAction");
        if (ActionType.gitAdd === action) {
            var filePath = options.filePath;
            try {
                var canAdd = true;
                // 检查文件/路径是否存在
                var targetFile = "" + path.normalize(path.join(this.projectDir, filePath));
                if (!fse.pathExistsSync(targetFile)) {
                    canAdd = false;
                    traceOutContent("\u6587\u4EF6/\u76EE\u5F55 \u4E0D\u5B58\u5728", "" + targetFile);
                }
                if (canAdd) {
                    require('child_process').execSync("git add -v \"" + filePath + "\"", {
                        cwd: "" + path.normalize(this.projectDir)
                    });
                }
            }
            catch (e) {
            }
        }
        traceEndCommand("commonGitAction");
    };
    PatchHelper.prototype.commonProcessForPackageJSONFile = function (_a) {
        var action = _a.action, options = _a.options;
        traceStartCommand("commonProcessForPackageJSONFile");
        var filePath = options.filePath;
        var saveOptions = {
            spaces: 2
        };
        traceOutContent("[JSONFile]", filePath);
        // 检查文件是否存在
        if (!fse.pathExistsSync(filePath)) {
            traceRunError(new Error(filePath + " is not exist!"));
            return;
        }
        // 解析JSON文件
        function _getJsonObj(reviverCallback) {
            if (reviverCallback === void 0) { reviverCallback = undefined; }
            var ObjectInfo = null;
            try {
                ObjectInfo = fse.readJsonSync(filePath);
            }
            catch (e) {
                traceRunError(e);
            }
            return ObjectInfo;
        }
        if (ActionType.contentAdd === action) {
            var keyPath = options.keyPath, keyValue = options.keyValue, overWrite = options.overWrite;
            var ObjectInfo = _getJsonObj(filePath);
            var canAdd = !!!_.has(ObjectInfo, keyPath) || !!overWrite;
            if (canAdd) {
                _.set(ObjectInfo, keyPath, keyValue);
                fse.writeJsonSync(filePath, ObjectInfo, saveOptions);
            }
        }
        else if (ActionType.contentReplace === action) {
            var keyPath = options.keyPath, keyValue = options.keyValue, match = options.match;
            var ObjectInfo = _getJsonObj(filePath);
            var canReplace = !!_.has(ObjectInfo, keyPath);
            if (canReplace) {
                var oldValue = _.get(ObjectInfo, keyPath);
                var newValue = oldValue;
                if (_.isUndefined(match)) {
                    newValue = keyValue;
                }
                else {
                    /// 根据不同类型进行解析处理
                    if (_.isPlainObject(oldValue)) {
                        try {
                            newValue = JSON.parse(JSON.stringify(oldValue).replace(match, keyValue));
                        }
                        catch (e) {
                        }
                    }
                    else if (_.isArray(oldValue)) {
                        try {
                            newValue = JSON.parse(JSON.stringify(oldValue).replace(match, keyValue));
                        }
                        catch (e) {
                        }
                    }
                    else if (_.isString(oldValue)) {
                        try {
                            newValue = oldValue.replace(match, keyValue);
                        }
                        catch (e) {
                        }
                    }
                    else if (_.isNumber(oldValue)) {
                        try {
                            newValue = ("" + oldValue).replace(match, keyValue);
                        }
                        catch (e) {
                        }
                    }
                    else if (_.isBoolean(oldValue)) {
                        if (oldValue !== keyValue) {
                            newValue = keyValue;
                        }
                    }
                }
                /// 设置新值
                _.set(ObjectInfo, keyPath, newValue);
                fse.writeJsonSync(filePath, ObjectInfo, saveOptions);
            }
        }
        else if (ActionType.contentReplaceX === action) {
            var match = options.match, content = options.content;
            /// TODO：使用文件替换的方法
        }
        else if (ActionType.contentRemove === action) {
            var keyPath = options.keyPath;
            var ObjectInfo = _getJsonObj(filePath);
            var canRemove = !!_.has(ObjectInfo, keyPath);
            if (canRemove) {
                _.unset(ObjectInfo, keyPath);
                fse.writeJsonSync(filePath, ObjectInfo, saveOptions);
            }
        }
        traceEndCommand("commonProcessForPackageJSONFile");
    };
    PatchHelper.prototype.commonProcessForNormalFile = function (_a) {
        var action = _a.action, options = _a.options;
        traceStartCommand("commonProcessForNormalFile");
        traceEndCommand("commonProcessForNormalFile");
    };
    PatchHelper.prototype.commonProcessCommand = function (command, opt) {
        if (opt === void 0) { opt = {}; }
        traceStartCommand("commonProcessCommand", true);
        var content = '';
        try {
            var buffer = require('child_process').execSync(command, __assign({ cwd: this.projectDir }, opt));
            if (util.isBuffer(buffer)) {
                content = buffer.toString('utf-8');
            }
            else if (util.isString(buffer)) {
                content = buffer;
            }
        }
        catch (e) {
        }
        return content;
    };
    PatchHelper.prototype.commonGetGitVersion = function () {
        traceStartCommand("commonGetGitVersion", true);
        // semantic versioning
        var version = {
            major: 0,
            minor: 0,
            patch: 0,
        };
        try {
            var content = this.commonProcessCommand("git version");
            // content like "git version 1.18.3.1.windows.1"
            var parts = content.split("."); // [ 'git version 1', '18', '3', '1' ]
            if (parts.length > 3) {
                var maybeMajorVersions = parts[0].split(" "); // [ 'git', 'version', '1' ]
                if (maybeMajorVersions.length > 2) {
                    version.major = parseInt(maybeMajorVersions[2]);
                }
                version.minor = parseInt(parts[1]);
                version.patch = parseInt(parts[2]);
            }
        }
        catch (e) {
        }
        return version;
    };
    PatchHelper.prototype.commonGetGitRemoteUrl = function (remoteName) {
        if (remoteName === void 0) { remoteName = 'origin'; }
        traceStartCommand("commonGetGitRemoteUrl", true);
        try {
            var specGitVersion = {
                major: 2,
                minor: 7,
                patch: 0,
                compare: function (other) {
                    var fn = function (first, last) {
                        var compare = -1;
                        first.every(function (value, index) {
                            var otherValue = 0;
                            if (index >= last.length) { // [1,1], [1]
                                otherValue = 0;
                            }
                            else {
                                otherValue = last[index];
                            }
                            if (value > otherValue) {
                                compare = 1;
                                return false;
                            }
                            else if ((value === otherValue) && (index === first.length - 1)) {
                                compare = 0;
                                return false;
                            }
                        });
                        return compare;
                    };
                    return fn([this.major, this.minor, this.patch], [other.major, other.minor, other.patch]);
                }
            };
            var curGitVersion = this.commonGetGitVersion();
            var compare = specGitVersion.compare(curGitVersion);
            // console.log(`specGitVersion, curGitVersion, versionDiff`, specGitVersion, curGitVersion, versionDiff);
            if (compare <= 0) {
                var content = this.commonProcessCommand("git remote get-url " + remoteName);
                return content.replace('.git\n', '/');
            }
            else {
                traceOutContent("Git\u7248\u672C\u68C0\u6D4B", "\u5F53\u524DGit\u7248\u672C\u4E0D\u652F\u6301 git remote get-url \u547D\u4EE4");
            }
        }
        catch (e) {
        }
        return '';
    };
    PatchHelper.prototype.commonFormatForPackageJSONFile = function (jsonFile, cwd) {
        traceStartCommand("commonFormatForPackageJSONFile", true);
        try {
            var cls = path.normalize(path.join(__dirname, 'node_modules', '.bin', 'format-package'));
            var command = "\"" + cls + "\" -w " + jsonFile;
            traceOutContent('[command]', command);
            traceOutContent('[cwd]', cwd);
            var result = require('child_process').execSync(command, {
                cwd: cwd
            });
            traceOutContent('[formatJSON]', result);
        }
        catch (e) {
            traceRunError(e);
        }
    };
    return PatchHelper;
}());
/**
 * 最外层，root包
 */
var RootProjectPatchHelper = /** @class */ (function (_super) {
    __extends(RootProjectPatchHelper, _super);
    function RootProjectPatchHelper() {
        var _this = this;
        var detectProjectPath = "";
        [
            // ../ = node_module; ../../ = wodax-project; ../../../ = packages; ../../../../ = root
            "" + path.normalize(path.join(__dirname, '../../../../')),
            // ../ = node_module; ../../ = root
            "" + path.normalize(path.join(__dirname, '../../'))
        ].forEach(function (onePath) {
            if (RootProjectPatchHelper.foundRootProject(onePath)) {
                detectProjectPath = onePath;
            }
        });
        _this = _super.call(this, {
            _name: "RootProjectPathHelper",
            _patchRootDir: "" + path.normalize(path.join(__dirname, PATCH_FILES_DIR_NAME + "/" + ROOT_PROJECT_PATCH_DIR_NAME + "/")),
            _projectDir: "" + detectProjectPath
        }) || this;
        return _this;
    }
    RootProjectPatchHelper.foundRootProject = function (projectDir) {
        traceStartCommand("RootProjectPatchHelper.foundRootProject");
        var found = false;
        [WODAX_PROJECT_DIR_NAME, WODAX_CORE_H5_PROJECT_DIR_NAME].forEach(function (subPrjName) {
            var dir = "" + path.normalize(path.join(projectDir, "" + PACKAGES_DIR_NAME, "" + subPrjName));
            if (enableDebug) {
                console.log("\u68C0\u6D4B\u8DEF\u5F84\u662F\u5426\u5B58\u5728?", "'" + dir + "'");
            }
            found = found || fse.pathExistsSync("" + dir);
        });
        traceEndCommand("RootProjectPatchHelper.foundRootProject");
        return found;
    };
    RootProjectPatchHelper.prototype.apply = function () {
        var _this = this;
        traceStartCommand("RootProjectPatchHelper.apply");
        /// 严格核查，当前是否包含 wodax-project 的子包的 root 包
        if (!RootProjectPatchHelper.foundRootProject(this.projectDir)) {
            traceWarning('[警告]', "\u6CA1\u6709\u627E\u5230\u7B26\u5408wodax-project\u9879\u76EE\u7ED3\u6784\u7684\u76EE\u5F55\u5168\u5C40\u5DE5\u7A0B\u5305(root)");
            traceEndCommand("RootProjectPatchHelper.apply");
            return;
        }
        /// Step： 对文件的操作
        {
            /// 通用操作：同步目录或者文件
            this.commonProcessForFolderFile({ action: ActionType.addOrUpdateFolderFile, options: {} });
            /// 删除操作：非通用，需要特别指定目录及文件
            [
            /// eg. yarn.lock
            ].forEach(function (onePath) {
                _this.commonProcessForFolderFile({ action: ActionType.deleteFolderFile, options: { filePath: onePath } });
            });
        }
        /// Step：对package.json文件的处理
        {
            /// 更新 package.json 中的 "fix:changelog",
            var defaultGitUrl = this.commonGetGitRemoteUrl();
            var specJsonFile_1 = "" + path.normalize(path.join(this.projectDir, 'package.json'));
            [
                {
                    action: ActionType.contentAdd,
                    keyPath: ['scripts', 'fix:changelog'],
                    keyValue: "replace \"http://git.woda.ink///\" \"" + defaultGitUrl + "\" CHANGELOG.md",
                    overWrite: true
                },
                /// 修复build, 根包要求主动更新build脚本等相关内容
                {
                    action: ActionType.contentAdd,
                    keyPath: ['scripts', 'build'],
                    keyValue: "npm run check-update && node ./scripts/build.js",
                    overWrite: true
                },
                /// 修复 check-update
                {
                    action: ActionType.contentAdd,
                    keyPath: ['scripts', 'check-update'],
                    keyValue: "node ./scripts/check_update.js",
                    overWrite: true
                },
                {
                    action: ActionType.contentAdd,
                    keyPath: ['scripts', 'prettier'],
                    keyValue: "node ./scripts/prettier.js",
                    overWrite: true
                },
                {
                    action: ActionType.contentAdd,
                    keyPath: ['scripts', 'release'],
                    keyValue: "standard-version",
                    overWrite: true
                },
                /// 针对devDependencies的处理
                {
                    action: ActionType.contentAdd,
                    keyPath: ['devDependencies', 'wodax-update-notifier'],
                    keyValue: getCurrentPackageVersion(),
                    overWrite: true
                },
            ].forEach(function (cfg) {
                _this.commonProcessForPackageJSONFile({
                    action: cfg.action,
                    options: __assign(__assign({}, cfg), { filePath: specJsonFile_1 })
                });
            });
            this.commonFormatForPackageJSONFile("package.json", this.projectDir);
        }
        /// TODO: 哪些文件内容需要替换
        /// TODO: 哪些文件内容需要添加
        /// TODO: 哪些文件内容需要删除
        /// Step：以下文件添加添加到git中
        {
            /// git操作：非通用，需要特别指定目录及文件
            [
                '.editorconfig',
                '.eslintignore',
                '.eslintrc.js',
                '.gitignore',
                '.npmrc',
                '.yarnrc',
                '.stylelintignore',
                '.versionrc',
                'commitlint.config.js',
                'tslint.json',
                'lerna.json',
                'package-lock.json',
                'yarn.lock',
                './scripts/build.js',
                './scripts/check_update.js',
            ].forEach(function (onePath) {
                _this.commonGitAction({ action: ActionType.gitAdd, options: { filePath: onePath } });
            });
        }
        traceEndCommand("RootProjectPatchHelper.apply");
    };
    return RootProjectPatchHelper;
}(PatchHelper));
/**
 * 主项目包： wodax-projerct
 * 与root包关系：属于子项目包
 * 位置： 例如：./packages/wodax-project/
 */
var WodaXProjectPatchHelper = /** @class */ (function (_super) {
    __extends(WodaXProjectPatchHelper, _super);
    function WodaXProjectPatchHelper() {
        var _this = this;
        var detectProjectPath = "";
        [
            // current is root project
            "" + path.normalize(path.join(__dirname, '../../', "" + PACKAGES_DIR_NAME, "" + WODAX_PROJECT_DIR_NAME)),
            // ../ = node_module; ../../ = current project
            "" + path.normalize(path.join(__dirname, '../../'))
        ].forEach(function (onePath) {
            if (WodaXProjectPatchHelper.foundWodaXProject(onePath)) {
                detectProjectPath = onePath;
            }
        });
        _this = _super.call(this, {
            _name: "WodaXProjectPatchHelper",
            _patchRootDir: "" + path.normalize(path.join(__dirname, PATCH_FILES_DIR_NAME + "/" + PACKAGES_DIR_NAME + "/" + WODAX_PROJECT_DIR_NAME)),
            _projectDir: "" + detectProjectPath
        }) || this;
        return _this;
    }
    WodaXProjectPatchHelper.foundWodaXProject = function (projectDir) {
        var dir = "" + path.normalize(projectDir);
        if (enableDebug) {
            console.log("check dir exist?", "'" + dir + "'");
        }
        return fse.pathExistsSync("" + dir) && (_.endsWith(dir, "" + path.normalize("/" + WODAX_PROJECT_DIR_NAME + "/")) ||
            _.endsWith(dir, "" + path.normalize("/" + WODAX_PROJECT_DIR_NAME)));
    };
    WodaXProjectPatchHelper.prototype.apply = function () {
        var _this = this;
        traceStartCommand("WodaXProjectPatchHelper.apply");
        /// 严格核查，当前设定的工作目录是否为wodax-project工程
        if (!WodaXProjectPatchHelper.foundWodaXProject(this.projectDir)) {
            traceWarning('[警告]', "\u6CA1\u6709\u627E\u5230\u7B26\u5408wodax-project\u9879\u76EE\u7ED3\u6784\u7684\u76EE\u5F55...");
            traceEndCommand("WodaXProjectPatchHelper.apply");
            return;
        }
        /// Step： 对文件的操作
        {
            /// 通用操作：同步目录或者文件
            this.commonProcessForFolderFile({ action: ActionType.addOrUpdateFolderFile, options: {} });
            /// 删除操作：非通用，需要特别指定目录及文件
            [
                'script/build.js',
                'script/preinstall.js',
                'package-lock.json' // 只允许yarn来安装提速，对于子项目，不建议使用npm来安装
                /// eg. yarn.lock
            ].forEach(function (onePath) {
                _this.commonProcessForFolderFile({ action: ActionType.deleteFolderFile, options: { filePath: onePath } });
            });
        }
        /// Step：对package.json文件的处理
        {
            /// 更新 package.json 中的 "fix:changelog",
            var defaultGitUrl = this.commonGetGitRemoteUrl();
            var specJsonFile_2 = "" + path.normalize(path.join(this.projectDir, 'package.json'));
            [
                /// 限制包安装
                {
                    action: ActionType.contentRemove,
                    keyPath: ['scripts', 'preinstall']
                },
                /// script 脚本处理
                {
                    action: ActionType.contentAdd,
                    keyPath: ['scripts', 'analyze'],
                    keyValue: "cross-env ANALYZE=1 umi build",
                    overWrite: true
                },
                {
                    action: ActionType.contentAdd,
                    keyPath: ['scripts', 'build'],
                    keyValue: "umi build",
                    overWrite: true
                },
                {
                    action: ActionType.contentAdd,
                    keyPath: ['scripts', 'check-update'],
                    keyValue: "node ./scripts/check_update.js",
                    overWrite: true
                },
                {
                    action: ActionType.contentAdd,
                    keyPath: ['scripts', 'checklist'],
                    keyValue: "yarn && yarn check-update && yarn lint:fix && yarn prettier && yarn lint",
                    overWrite: true
                },
                {
                    action: ActionType.contentAdd,
                    keyPath: ['scripts', 'commit'],
                    keyValue: "git-cz",
                    overWrite: true
                },
                {
                    action: ActionType.contentAdd,
                    keyPath: ['scripts', 'start'],
                    keyValue: "yarn dev",
                    overWrite: true
                },
                {
                    action: ActionType.contentAdd,
                    keyPath: ['scripts', 'dev'],
                    keyValue: "yarn checklist && umi dev",
                    overWrite: true
                },
                {
                    action: ActionType.contentAdd,
                    keyPath: ['scripts', 'dev:quick'],
                    keyValue: "umi dev",
                    overWrite: true
                },
                {
                    action: ActionType.contentRemove,
                    keyPath: ['scripts', 'dev-quick']
                },
                {
                    action: ActionType.contentAdd,
                    keyPath: ['scripts', 'dev:no-mock'],
                    keyValue: "yarn checklist && cross-env MOCK=none umi dev",
                    overWrite: true
                },
                {
                    action: ActionType.contentRemove,
                    keyPath: ['scripts', 'fix:changelog']
                },
                {
                    action: ActionType.contentAdd,
                    keyPath: ['scripts', 'fix:changelog'],
                    keyValue: "replace \"http://git.woda.ink///\" \"" + defaultGitUrl + "\" CHANGELOG.md",
                    overWrite: true
                },
                {
                    action: ActionType.contentAdd,
                    keyPath: ['scripts', 'lint'],
                    keyValue: "yarn lint:script && yarn lint:style",
                    overWrite: true
                },
                {
                    action: ActionType.contentAdd,
                    keyPath: ['scripts', 'lint-staged'],
                    keyValue: "lint-staged",
                    overWrite: true
                },
                {
                    action: ActionType.contentRemove,
                    keyPath: ['scripts', 'lint-staged:js']
                },
                {
                    action: ActionType.contentAdd,
                    keyPath: ['scripts', 'lint:es'],
                    keyValue: "eslint --ext .js,.jsx,.ts,.tsx ./src",
                    overWrite: true
                },
                {
                    action: ActionType.contentAdd,
                    keyPath: ['scripts', 'lint:fix'],
                    keyValue: "yarn lint:fix:es && yarn lint:fix:ts",
                    overWrite: true
                },
                {
                    action: ActionType.contentAdd,
                    keyPath: ['scripts', 'lint:fix:es'],
                    keyValue: "eslint --fix --ext .js,.jsx,.ts,.tsx ./src",
                    overWrite: true
                },
                {
                    action: ActionType.contentAdd,
                    keyPath: ['scripts', 'lint:fix:ts'],
                    keyValue: "tslint --fix -c ./tslint.json './**/*{.ts,*tsx}'",
                    overWrite: true
                },
                {
                    action: ActionType.contentAdd,
                    keyPath: ['scripts', 'lint:script'],
                    keyValue: "yarn lint:es && yarn lint:ts",
                    overWrite: true
                },
                {
                    action: ActionType.contentAdd,
                    keyPath: ['scripts', 'lint:style'],
                    keyValue: "node ./scripts/lint_style.js",
                    overWrite: true
                },
                {
                    action: ActionType.contentAdd,
                    keyPath: ['scripts', 'lint:ts'],
                    keyValue: "tslint -c ./tslint.json './**/*{.ts,*tsx}'",
                    overWrite: true
                },
                {
                    action: ActionType.contentAdd,
                    keyPath: ['scripts', 'prettier'],
                    keyValue: "node ./scripts/prettier.js",
                    overWrite: true
                },
                {
                    action: ActionType.contentAdd,
                    keyPath: ['scripts', 'release'],
                    keyValue: "standard-version",
                    overWrite: true
                },
                {
                    action: ActionType.contentAdd,
                    keyPath: ['scripts', 'test'],
                    keyValue: "umi test",
                    overWrite: true
                },
                {
                    action: ActionType.contentAdd,
                    keyPath: ['scripts', 'test:all'],
                    keyValue: "node ./tests/run-tests.js",
                    overWrite: true
                },
                {
                    action: ActionType.contentAdd,
                    keyPath: ['scripts', 'test:component'],
                    keyValue: "umi test ./src/components",
                    overWrite: true
                },
                /// 删除 "lint-staged"
                {
                    action: ActionType.contentRemove,
                    keyPath: ['lint-staged'],
                },
                /// husky 处理
                {
                    action: ActionType.contentAdd,
                    keyPath: ['husky', 'hooks', 'pre-commit'],
                    keyValue: "lint-staged -d",
                    overWrite: true
                },
                {
                    action: ActionType.contentRemove,
                    keyPath: ['husky', 'hooks', 'pre-push']
                },
                {
                    action: ActionType.contentAdd,
                    keyPath: ['husky', 'hooks', 'post-commit'],
                    keyValue: "git update-index --again",
                    overWrite: true
                },
                /// standard-version 处理
                {
                    action: ActionType.contentAdd,
                    keyPath: ['standard-version', 'scripts', 'postchangelog'],
                    keyValue: "yarn fix:changelog",
                    overWrite: true
                },
                /// 针对devDependencies的处理
                {
                    action: ActionType.contentAdd,
                    keyPath: ['devDependencies', 'replace'],
                    keyValue: "^1.1.1"
                },
                {
                    action: ActionType.contentAdd,
                    keyPath: ['devDependencies', 'rimraf'],
                    keyValue: "^3.0.0"
                },
                {
                    action: ActionType.contentAdd,
                    keyPath: ['devDependencies', 'mockjs'],
                    keyValue: "^1.1.0"
                },
                {
                    action: ActionType.contentAdd,
                    keyPath: ['devDependencies', 'wodax-update-notifier'],
                    keyValue: getCurrentPackageVersion(),
                    overWrite: true,
                },
                /// 针对eslint_React-Hooks规则的处理, 先预处理，后期上新
                {
                    action: ActionType.contentAdd,
                    keyPath: ['devDependencies', 'eslint-plugin-react-hooks'],
                    keyValue: "^2.3.0",
                    overWrite: false,
                },
            ].forEach(function (cfg) {
                _this.commonProcessForPackageJSONFile({
                    action: cfg.action,
                    options: __assign(__assign({}, cfg), { filePath: specJsonFile_2 })
                });
            });
            this.commonFormatForPackageJSONFile("package.json", this.projectDir);
        }
        /// Step：以下文件添加添加到git中
        {
            /// git操作：非通用，需要特别指定目录及文件
            [
                /// eg. yarn.lock
                '.editorconfig',
                '.env',
                '.eslintignore',
                '.eslintrc.js',
                '.gitignore',
                '.lintstagedrc.js',
                '.npmrc',
                '.prettierignore',
                '.prettierrc',
                '.stylelintrc.json',
                '.versionrc',
                '.yarnrc',
                'commitlint.config.js',
                'jest.config.js',
                'jest-puppeteer.config.js',
                'jsconfig.json',
                'tsconfig.json',
                'tslint.json',
                'yarn.lock',
                'webpack.config.js',
                './scripts/check_update.js',
                './scripts/lint_style.js',
                './scripts/prettier.js',
            ].forEach(function (onePath) {
                _this.commonGitAction({ action: ActionType.gitAdd, options: { filePath: onePath } });
            });
        }
        traceEndCommand("WodaXProjectPatchHelper.apply");
    };
    return WodaXProjectPatchHelper;
}(PatchHelper));
/**
 * H5主项目包：wodax-core-h5
 * 与root包关系：属于子项目包
 * 位置： 例如：./packages/wodax-core-h5/
 */
var WodaXH5ProjectPatchHelper = /** @class */ (function (_super) {
    __extends(WodaXH5ProjectPatchHelper, _super);
    function WodaXH5ProjectPatchHelper() {
        var _this = this;
        var detectProjectPath = "";
        [
            // current is root project
            "" + path.normalize(path.join(__dirname, '../../', "" + PACKAGES_DIR_NAME, "" + WodaXH5ProjectPatchHelper.PROJECT_DIR_NAME)),
            // ../ = node_module; ../../ = current project
            "" + path.normalize(path.join(__dirname, '../../'))
        ].forEach(function (onePath) {
            if (WodaXH5ProjectPatchHelper.foundCoreH5Project(onePath)) {
                detectProjectPath = onePath;
            }
        });
        _this = _super.call(this, {
            _name: "WodaXH5ProjectPatchHelper",
            _patchRootDir: "" + path.normalize(path.join(__dirname, PATCH_FILES_DIR_NAME + "/" + PACKAGES_DIR_NAME + "/" + WodaXH5ProjectPatchHelper.PROJECT_DIR_NAME)),
            _projectDir: "" + detectProjectPath
        }) || this;
        return _this;
    }
    WodaXH5ProjectPatchHelper.foundCoreH5Project = function (projectDir) {
        var dir = "" + path.normalize(projectDir);
        if (enableDebug) {
            console.log("check dir exist?", "'" + dir + "'");
        }
        return fse.pathExistsSync("" + dir) && (_.endsWith(dir, "" + path.normalize("/" + WodaXH5ProjectPatchHelper.PROJECT_DIR_NAME + "/")) ||
            _.endsWith(dir, "" + path.normalize("/" + WodaXH5ProjectPatchHelper.PROJECT_DIR_NAME)));
    };
    WodaXH5ProjectPatchHelper.prototype.apply = function () {
        var _this = this;
        traceStartCommand("WodaXH5ProjectPatchHelper.apply");
        /// 严格核查，当前设定的工作目录是否为wodax-project工程
        if (!WodaXH5ProjectPatchHelper.foundCoreH5Project(this.projectDir)) {
            traceWarning('[警告]', "\u6CA1\u6709\u627E\u5230\u7B26\u5408" + WodaXH5ProjectPatchHelper.PROJECT_DIR_NAME + "\u9879\u76EE\u7ED3\u6784\u7684\u76EE\u5F55...");
            traceEndCommand("WodaXH5ProjectPatchHelper.apply");
            return;
        }
        /// Step： 对文件的操作
        {
            /// 通用操作：同步目录或者文件
            this.commonProcessForFolderFile({ action: ActionType.addOrUpdateFolderFile, options: {} });
            /// 删除操作：非通用，需要特别指定目录及文件
            [
                'script/build.js',
                'package-lock.json' // 只允许yarn来安装提速，对于子项目，不建议使用npm来安装
                /// eg. yarn.lock
            ].forEach(function (onePath) {
                _this.commonProcessForFolderFile({ action: ActionType.deleteFolderFile, options: { filePath: onePath } });
            });
        }
        /// Step：对package.json文件的处理
        {
            var specJsonFile_3 = "" + path.normalize(path.join(this.projectDir, 'package.json'));
            [
                /// 针对模板的处理
                {
                    action: ActionType.contentAdd,
                    keyPath: ['wodax', 'templateClassId'],
                    keyValue: "01-wodax-h5-app",
                },
                {
                    action: ActionType.contentAdd,
                    keyPath: ['wodax', 'templateKey'],
                    keyValue: "wh5",
                },
                {
                    action: ActionType.contentAdd,
                    keyPath: ['wodax', 'templateVersion'],
                    keyValue: "1.0.0",
                },
                /// script 脚本处理
                {
                    action: ActionType.contentAdd,
                    keyPath: ['scripts', 'start'],
                    keyValue: "if-env NODE_ENV=production && npm run -s serve || npm run -s dev",
                    overWrite: true
                },
                {
                    action: ActionType.contentAdd,
                    keyPath: ['scripts', 'build'],
                    keyValue: "preact build --no-prerender --optimize-minimize --dest dist --template src/index.html"
                },
                {
                    action: ActionType.contentAdd,
                    keyPath: ['scripts', 'serve:no-prerender'],
                    keyValue: "preact build --no-prerender --optimize-minimize && serve build --single",
                    overWrite: true
                },
                {
                    action: ActionType.contentAdd,
                    keyPath: ['scripts', 'serve:release'],
                    keyValue: "preact build  --prerenderUrls ./scripts/prerender-urls.js --optimize-minimize && serve build --single",
                    overWrite: true
                },
                {
                    action: ActionType.contentAdd,
                    keyPath: ['scripts', 'serve'],
                    keyValue: "yarn checklist && preact build --optimize-minimize --template src/index.html && serve build",
                    overWrite: true
                },
                {
                    action: ActionType.contentAdd,
                    keyPath: ['scripts', 'dev'],
                    keyValue: "yarn checklist && preact watch --template src/index.html",
                    overWrite: true
                },
                {
                    action: ActionType.contentAdd,
                    keyPath: ['scripts', 'check-update'],
                    keyValue: "node ./scripts/check_update.js",
                    overWrite: true
                },
                {
                    action: ActionType.contentAdd,
                    keyPath: ['scripts', 'checklist'],
                    keyValue: "yarn && yarn check-update",
                    overWrite: true
                },
                {
                    action: ActionType.contentAdd,
                    keyPath: ['scripts', 'lint'],
                    keyValue: "eslint src",
                    overWrite: true
                },
                {
                    action: ActionType.contentAdd,
                    keyPath: ['scripts', 'test'],
                    keyValue: "jest",
                    overWrite: true
                },
                {
                    action: ActionType.contentAdd,
                    keyPath: ['scripts', 'coverage'],
                    keyValue: "jest --coverage",
                    overWrite: true
                },
                {
                    action: ActionType.contentAdd,
                    keyPath: ['scripts', 'analyze'],
                    keyValue: "preact build --analyze --template src/index.html",
                    overWrite: true
                },
                /// 针对dependencies的处理
                {
                    action: ActionType.contentRemove,
                    keyPath: ['dependencies', 'react-router-dom']
                },
                {
                    action: ActionType.contentRemove,
                    keyPath: ['dependencies', 'wodax-react-router-cache']
                },
                {
                    action: ActionType.contentAdd,
                    keyPath: ['dependencies', 'wodax-react-virtuoso-ex'],
                    keyValue: "^1.0.0"
                },
                {
                    action: ActionType.contentAdd,
                    keyPath: ['dependencies', 'mobx'],
                    keyValue: "4.15.1",
                    overWrite: true
                },
            ].forEach(function (cfg) {
                _this.commonProcessForPackageJSONFile({
                    action: cfg.action,
                    options: __assign(__assign({}, cfg), { filePath: specJsonFile_3 })
                });
            });
        }
        /// Step：以下文件添加添加到git中
        {
            /// git操作：非通用，需要特别指定目录及文件
            [
                /// eg. yarn.lock
                '.babelrc',
                '.editorconfig',
                '.gitignore',
                '.yarnrc',
                'yarn.lock',
                'preact.config.js',
                './scripts/check_update.js',
            ].forEach(function (onePath) {
                _this.commonGitAction({ action: ActionType.gitAdd, options: { filePath: onePath } });
            });
        }
        traceEndCommand("WodaXH5ProjectPatchHelper.apply");
    };
    WodaXH5ProjectPatchHelper.PROJECT_DIR_NAME = WODAX_CORE_H5_PROJECT_DIR_NAME;
    return WodaXH5ProjectPatchHelper;
}(PatchHelper));
/// 尝试应用所有补丁
function tryApplyPatches(debug) {
    if (debug === void 0) { debug = false; }
    try {
        traceOutContent('>>> ( ఠൠఠ )ﾉ', '执行补丁处理程序 ...');
        enableDebug = debug;
        var rootPrjPatchHelper = new RootProjectPatchHelper();
        var wodaPrjPatchHelper = new WodaXProjectPatchHelper();
        var wodaH5PrjPatchHelper = new WodaXH5ProjectPatchHelper();
        rootPrjPatchHelper.apply();
        wodaPrjPatchHelper.apply();
        wodaH5PrjPatchHelper.apply();
    }
    catch (e) {
        traceOutContent('>>> ( ఠൠఠ )ﾉ', '补丁处理程序执行出错:');
        console.trace(e);
    }
    finally {
        traceOutContent('>>> ( ఠൠఠ )ﾉ', '补丁处理程序完成 ...');
    }
}
exports.tryApplyPatches = tryApplyPatches;
//# sourceMappingURL=patch.js.map